library clickzinfluttersdk;

import 'package:android_play_install_referrer/android_play_install_referrer.dart';
import 'dart:async';

import 'package:http/http.dart' as http;
import 'package:shared_preferences/shared_preferences.dart';

class ClickzinTracker {
  static const String _API_URL =
      "https://kalpssoft.clickzin.com/postback?uid=#UID#&event=#EVENT#";
  static const String _UTM_CAMPAIGN = "utm_campaign";
  static const String _UTM_SOURCE = "utm_source";
  static const String _UTM_MEDIUM = "utm_medium";
  static const String _UTM_TERM = "utm_term";
  static const String _UTM_CONTENT = "utm_content";
  static const String _UTM_UID = "utm_uid";
  static const String _UTM_NETWORK = "utm_network";
  static const String _UTM_REFER = "utm_refer";

  static final ClickzinTracker _singleton = ClickzinTracker._internal();
  late String _appKey;
  late bool _enableLogs;

  /// Create instance of clickzin with appKey as provided by the Marketing partner.
  factory ClickzinTracker(String appKey, bool enableLogs) {
    _singleton._appKey = appKey;
    _singleton._enableLogs = enableLogs;
    // Obtain shared preferences.
    return _singleton;
  }

  ClickzinTracker._internal();

  /// Start tracking with initial event.
  Future<void> startTracking(String initialEvent) async {
    if (_appKey.isEmpty) {
      throw Exception("Invalid appKey");
    }
    // Obtain shared preferences.
    final prefs = await SharedPreferences.getInstance();

    // ReferrerDetails referrerDetails =
    //     await AndroidPlayInstallReferrer.installReferrer;

    String? query =
        "utm_source%3Dclickzin%26utm_medium%3D{AFFID}%26utm_term%3DaffName={AFFNAME}%26utm_content%3D{SOURCE}%26utm_campaign%3D{OID}%26utm_uid%3D1234567890%26utm_network%3D{AFFID}%26utm_refer%3D{REFER_VALUE}";
    //referrerDetails.installReferrer;

    printLogs("query : $query");

    if (query != null) {
      _filterParams(query, prefs);
      await _processData(prefs, initialEvent);
    }
  }

  /// Tracking events like register, login and so on
  Future<void> trackEvent(String event) async {
    final prefs = await SharedPreferences.getInstance();
    await _processData(prefs, event);
  }

  Map<String, String> _filterParams(
      String query, SharedPreferences preferences) {
    Map<String, String> queryPairs = <String, String>{};

    var decodedQueries = Uri.decodeFull(query);

    printLogs("decodedQueries : $decodedQueries");

    List<String> pairs = decodedQueries.split("&");

    printLogs("pairs : $pairs");
    for (var pair in pairs) {
      if (pair.contains("=")) {
        List<String> params = pair.split("=");
        printLogs("params : $params and params length :${params.length}");
        if (params.isNotEmpty) {
          try {
            String key = params.first;
            String value = params.last;
            queryPairs.putIfAbsent(key, () => value);
            preferences.setString(key, value);
            printLogs("_filterParams : $key & value:$value");
            //Store data in preferences
          } catch (e) {
            printLogs("_filterParams exception : $e");
          }
        }
      }
    }
    return queryPairs;
  }

  Future<void> _processData(
      SharedPreferences prefs, String initialEvent) async {
    String? uid = prefs.getString(_UTM_UID);
    String? source = prefs.getString(_UTM_SOURCE);

    bool? isEventExecutedAlready = prefs.getBool(initialEvent);
    isEventExecutedAlready ??= false;

    printLogs("isEventExecutedAlready : $isEventExecutedAlready");
    if (source != null &&
        source.contains("clickzin") &&
        uid != null &&
        initialEvent.isNotEmpty &&
        !isEventExecutedAlready) {
      var response = await _sendRequest(uid, initialEvent);

      if (response != null && response == "Success") {
        prefs.setBool(initialEvent, true);
      }
    }
  }

  Future<String?> _sendRequest(String uid, String event) async {
    try {
      printLogs("uid : $uid");
      printLogs("event : $event");

      String url = _API_URL.replaceAll("#UID#", uid);
      url = url.replaceAll("#EVENT#", event);

      printLogs("_sendRequest url: $url");
      http.Response response = await http.get(Uri.parse(url));
      if (response.statusCode != 200) {
        return null;
      }

      printLogs(response);

      printLogs("Status Code : ${response.statusCode}");
      printLogs("Status response : ${response.body}");

      return response.body;
    } catch (e) {
      printLogs(" send request  error : ${e}");
      return null;
    }
  }

  void printLogs(Object message) {
    if (_enableLogs) {
      print(message);
    }
  }
}
